﻿using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Globalization;

namespace PlayingCardCreator
{
    public partial class MainWindow : Window
    {
        const int WIDTH = 80;
        const int HEIGHT = 112;

        const int X1 = 25;
        const int X2 = WIDTH / 2;
        const int X3 = WIDTH - X1;

        const int Y1 = 20;
        const int Y5 = HEIGHT / 2;
        const int Y9 = HEIGHT - Y1;

        // just for 7 and 8
        const int Y3 = (Y1 + Y5) / 2;
        const int Y7 = (Y5 + Y9) / 2;

        // just for 9 and 10
        const int Y4 = (2 * Y1 + Y9) / 3;
        const int Y6 = (Y1 + 2 * Y9) / 3;

        // just for 10
        const int Y2 = (Y1 + Y4) / 2;
        const int Y8 = (Y6 + Y9) / 2;

        string[] rankName = { "A", "2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K" };
        string[] suitChar = { "\x2660", "\x2663", "\x2665", "\x2666" };
        FontFamily fontFamily = new FontFamily("Times New Roman");
        double cornerSize = 14;
        double innerSize = 30;
        double aceSize = 90;
        double innerTextSize = 72;
        Point[][] points = { new Point[] { new Point(X2, Y5) },
                             new Point[] { new Point(X2, Y1), new Point(X2, Y9) },
                             new Point[] { new Point(X2, Y1), new Point(X2, Y5), new Point(X2, Y9) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y9), new Point(X3, Y9) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X2, Y5), new Point(X1, Y9), new Point(X3, Y9) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y5), new Point(X3, Y5), new Point(X1, Y9), new Point(X3, Y9) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y5), new Point(X3, Y5), new Point(X1, Y9), new Point(X3, Y9), new Point(X2, Y3) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y5), new Point(X3, Y5), new Point(X1, Y9), new Point(X3, Y9), new Point(X2, Y3), new Point(X2, Y7) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y4), new Point(X3, Y4), new Point(X1, Y6), new Point(X3, Y6), new Point(X1, Y9), new Point(X3, Y9), new Point(X2, Y5) },
                             new Point[] { new Point(X1, Y1), new Point(X3, Y1), new Point(X1, Y4), new Point(X3, Y4), new Point(X1, Y6), new Point(X3, Y6), new Point(X1, Y9), new Point(X3, Y9), new Point(X2, Y2), new Point(X2, Y8) }
                           };


        public MainWindow()
        {
            InitializeComponent();

            for (int suit = 0; suit < 4; suit++)
            {
                double yBase = HEIGHT * suit;

                for (int rank = 0; rank < 13; rank++)
                {
                    double xBase = WIDTH * rank;

                    Border border = new Border
                    {
                        Width = WIDTH,
                        Height = HEIGHT,
                        Background = Brushes.White,
                        BorderThickness = new Thickness(1),
                        BorderBrush = Brushes.Black,
                        CornerRadius = new CornerRadius(6)
                    };

                    Canvas.SetLeft(border, xBase);
                    Canvas.SetTop(border, yBase);
                    canvas.Children.Add(border);

                    Brush brush = suit < 2 ? Brushes.Black : Brushes.Red;
                    RotateTransform rotate = new RotateTransform();
                    double multiplier = 1;

                    FormattedText formtxt = new FormattedText("10", CultureInfo.CurrentCulture, FlowDirection.LeftToRight,
                                                               new Typeface(fontFamily.Source), cornerSize, Brushes.Black);
                    double maxWidth = formtxt.Width;

                    for (int corner = 0; corner < 2; corner++)
                    {
                        TransformGroup xformGroup = new TransformGroup();
                        xformGroup.Children.Add(rotate);
                        xformGroup.Children.Add(new TranslateTransform(multiplier * 3, multiplier * 2));

                        TextBlock txtblkRankName = new TextBlock
                        {
                            FontFamily = fontFamily,
                            FontSize = cornerSize,
                            Text = rankName[rank],
                            Foreground = brush,
                            RenderTransform = xformGroup,
                            Width = maxWidth,
                            TextAlignment = TextAlignment.Center
                        };

                        Canvas.SetLeft(txtblkRankName, xBase);
                        Canvas.SetTop(txtblkRankName, yBase);
                        canvas.Children.Add(txtblkRankName);

                        xformGroup = new TransformGroup();
                        xformGroup.Children.Add(rotate);
                        xformGroup.Children.Add(new TranslateTransform(multiplier * 3, multiplier * 14));

                        TextBlock txtblkSuit = new TextBlock
                        {
                            FontFamily = fontFamily,
                            FontSize = cornerSize,
                            Text = suitChar[suit],
                            Foreground = brush,
                            RenderTransform = xformGroup,
                            Width = maxWidth,
                            TextAlignment = TextAlignment.Center
                        };

                        Canvas.SetLeft(txtblkSuit, xBase);
                        Canvas.SetTop(txtblkSuit, yBase);
                        canvas.Children.Add(txtblkSuit);

                        rotate = new RotateTransform(180, WIDTH / 2, HEIGHT / 2);
                        multiplier = -1;
                    }


                    if (rank < points.GetLength(0))
                    {
                        double size = rank == 0 ? aceSize : innerSize;

                        formtxt = new FormattedText(suitChar[suit], CultureInfo.CurrentCulture, FlowDirection.LeftToRight,
                                                    new Typeface(fontFamily.Source), size, Brushes.Black);

                        for (int num = 0; num < rank + 1; num++)
                        {
                            TextBlock txtblkInner = new TextBlock
                            {
                                FontFamily = fontFamily,
                                FontSize = size,
                                Text = suitChar[suit],
                                Foreground = brush,
                            };

                            Point pt = points[rank][num];

                            if (pt.Y > Y5)
                            {
                                txtblkInner.RenderTransform = new RotateTransform(180, formtxt.Width / 2, formtxt.Height / 2);
                            }

                            Canvas.SetLeft(txtblkInner, xBase + pt.X - formtxt.Width / 2);
                            Canvas.SetTop(txtblkInner, yBase + pt.Y - formtxt.Height / 2);
                            canvas.Children.Add(txtblkInner);
                        }
                    }
                    else
                    {
                        formtxt = new FormattedText(rankName[rank], CultureInfo.CurrentCulture, FlowDirection.LeftToRight,
                                                    new Typeface(fontFamily.Source), innerTextSize, Brushes.Black);

                        TextBlock txtblkInner = new TextBlock
                        {
                            FontFamily = fontFamily,
                            FontSize = innerTextSize,
                            Text = rankName[rank],
                            Foreground = brush,
                        };

                        Canvas.SetLeft(txtblkInner, xBase + WIDTH / 2 - formtxt.Width / 2);
                        Canvas.SetTop(txtblkInner, yBase + HEIGHT / 2 - formtxt.Height / 2);
                        canvas.Children.Add(txtblkInner);
                    }
                }
            }
        }

        bool expanded;

        private void innerGrid_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            expanded ^= true;
            double scale = expanded ? 3.2 : 1;
            innerGrid.LayoutTransform = new ScaleTransform(scale, scale);
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            RenderTargetBitmap bitmap = new RenderTargetBitmap(13 * WIDTH, 4 * HEIGHT, 96, 96, PixelFormats.Default);
            bitmap.Render(canvas);

            FileStream stream = new FileStream("cards.png", FileMode.Create);
            PngBitmapEncoder pngEnc = new PngBitmapEncoder();
            pngEnc.Frames.Add(BitmapFrame.Create(bitmap));
            pngEnc.Save(stream);

            base.OnClosing(e);
        }
    }
}
